@@ -20,6 +20,7 @@
 
 #include "appearance.h"
 #include "gnome-wp-item.h"
+#include <gio/gio.h>
 #include <string.h>
 #include <libxml/parser.h>
 
@@ -33,7 +34,7 @@ static gboolean gnome_wp_xml_get_bool (const xmlNode * parent,
 
   prop = xmlGetProp ((xmlNode *) parent, (xmlChar*)prop_name);
   if (prop != NULL) {
-    if (!g_strcasecmp ((gchar *)prop, "true") || !g_strcasecmp ((gchar *)prop, "1")) {
+    if (!g_ascii_strcasecmp ((gchar *)prop, "true") || !g_ascii_strcasecmp ((gchar *)prop, "1")) {
       ret_val = TRUE;
     } else {
       ret_val = FALSE;
@@ -117,6 +118,8 @@ static void gnome_wp_xml_load_xml (AppearanceData *data,
     if (!strcmp ((gchar *)list->name, "wallpaper")) {
       GnomeWPItem * wp;
       gchar *pcolor = NULL, *scolor = NULL;
+      gchar *s;
+      gboolean have_scale = FALSE, have_shade = FALSE;
 
       wp = g_new0 (GnomeWPItem, 1);
 
@@ -128,7 +131,7 @@ static void gnome_wp_xml_load_xml (AppearanceData *data,
 	    const char * none = "(none)";
 	    gchar *content = g_strstrip ((gchar *)wpa->last->content);
 
-	    if (!strncmp (content, none, strlen (none)))
+	    if (!strcmp (content, none))
 	      wp->filename = g_strdup (content);
 	    else if (g_utf8_validate (content, -1, NULL) &&
 		     g_file_test (content, G_FILE_TEST_EXISTS))
@@ -160,14 +163,13 @@ static void gnome_wp_xml_load_xml (AppearanceData *data,
 	  }
 	} else if (!strcmp ((gchar *)wpa->name, "options")) {
 	  if (wpa->last != NULL) {
-	    wp->options = g_strdup (g_strstrip ((gchar *)wpa->last->content));
-	  } else {
-	    wp->options = gconf_client_get_string (data->client,
-						   WP_OPTIONS_KEY, NULL);
+	    wp->options = wp_item_string_to_option (g_strstrip ((gchar *)wpa->last->content));
+	    have_scale = TRUE;
 	  }
 	} else if (!strcmp ((gchar *)wpa->name, "shade_type")) {
 	  if (wpa->last != NULL) {
-	    wp->shade_type = g_strdup (g_strstrip ((gchar *)wpa->last->content));
+	    wp->shade_type = wp_item_string_to_shading (g_strstrip ((gchar *)wpa->last->content));
+	    have_shade = TRUE;
 	  }
 	} else if (!strcmp ((gchar *)wpa->name, "pcolor")) {
 	  if (wpa->last != NULL) {
@@ -195,10 +197,18 @@ static void gnome_wp_xml_load_xml (AppearanceData *data,
       }
 
       /* Verify the colors and alloc some GdkColors here */
-      if (wp->shade_type == NULL) {
-	wp->shade_type = gconf_client_get_string (data->client,
-						  WP_SHADING_KEY, NULL);
+      if (!have_scale) {
+        s = gconf_client_get_string (data->client, WP_OPTIONS_KEY, NULL);
+        wp->options = wp_item_string_to_option (s);
+        g_free (s);
       }
+
+      if (!have_shade) {
+        s = gconf_client_get_string (data->client, WP_SHADING_KEY, NULL);
+        wp->shade_type = wp_item_string_to_shading (s);
+        g_free (s);
+      }
+
       if (pcolor == NULL) {
 	pcolor = gconf_client_get_string (data->client,
 					  WP_PCOLOR_KEY, NULL);
@@ -237,27 +247,93 @@ static void gnome_wp_xml_load_xml (AppearanceData *data,
   xmlFreeDoc (wplist);
 }
 
-static void gnome_wp_file_changed (GnomeVFSMonitorHandle * handle,
-				   const gchar * monitor_uri,
-				   const gchar * info_uri,
-				   GnomeVFSMonitorEventType event_type,
-				   AppearanceData *data) {
+static void gnome_wp_file_changed (GFileMonitor *monitor,
+                                   GFile *file,
+                                   GFile *other_file,
+                                   GFileMonitorEvent event_type,
+                                   AppearanceData *data) {
   gchar * filename;
 
   switch (event_type) {
-  case GNOME_VFS_MONITOR_EVENT_CHANGED:
-  case GNOME_VFS_MONITOR_EVENT_CREATED:
-    filename = gnome_vfs_get_local_path_from_uri (info_uri);
+  case G_FILE_MONITOR_EVENT_CHANGED:
+  case G_FILE_MONITOR_EVENT_CREATED:
+    filename = g_file_get_path (file);
     gnome_wp_xml_load_xml (data, filename);
     g_free (filename);
+    break;
   default:
     break;
   }
 }
 
+static void gnome_wp_xml_add_monitor (GFile *directory,
+                                      AppearanceData *data) {
+  GFileMonitor *monitor;
+  GError *error = NULL;
+
+  monitor = g_file_monitor_directory (directory,
+                                      G_FILE_MONITOR_NONE,
+                                      NULL,
+                                      &error);
+  if (error != NULL) {
+    gchar *path;
+
+    path = g_file_get_parse_name (directory);
+    g_warning ("Unable to monitor directory %s: %s",
+               path, error->message);
+    g_error_free (error);
+    g_free (path);
+    return;
+  }
+
+  g_signal_connect (monitor, "changed",
+                    G_CALLBACK (gnome_wp_file_changed),
+                    data);
+}
+
+static void gnome_wp_xml_load_from_dir (const gchar *path,
+                                        AppearanceData *data) {
+  GFile *directory;
+  GFileEnumerator *enumerator;
+  GError *error = NULL;
+  GFileInfo *info;
+
+  if (!g_file_test (path, G_FILE_TEST_IS_DIR)) {
+    return;
+  }
+
+  directory = g_file_new_for_path (path);
+  enumerator = g_file_enumerate_children (directory,
+                                          G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                          G_FILE_QUERY_INFO_NONE,
+                                          NULL,
+                                          &error);
+  if (error != NULL) {
+    g_warning ("Unable to check directory %s: %s", path, error->message);
+    g_error_free (error);
+    g_object_unref (directory);
+    return;
+  }
+
+  while ((info = g_file_enumerator_next_file (enumerator, NULL, NULL))) {
+    const gchar *filename;
+    gchar *fullpath;
+
+    filename = g_file_info_get_name (info);
+    fullpath = g_build_filename (path, filename, NULL);
+    g_object_unref (info);
+
+    gnome_wp_xml_load_xml (data, fullpath);
+    g_free (fullpath);
+  }
+  g_file_enumerator_close (enumerator, NULL, NULL);
+
+  gnome_wp_xml_add_monitor (directory, data);
+
+  g_object_unref (directory);
+}
+
 void gnome_wp_xml_load_list (AppearanceData *data) {
-  GnomeVFSMonitorHandle * handle;
-  GList * list, * l;
   const gchar * xdgdirslist;
   gchar * wpdbfile;
   gchar ** xdgdirs;
@@ -292,54 +368,12 @@ void gnome_wp_xml_load_list (AppearanceData *data) {
 
     datadir = g_build_filename (xdgdirs[i], "gnome-background-properties",
 				NULL);
-    if (g_file_test (datadir, G_FILE_TEST_IS_DIR)) {
-      gnome_vfs_directory_list_load (&list, datadir,
-				     GNOME_VFS_FILE_INFO_DEFAULT |
-				     GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-
-      for (l = list; l != NULL; l = l->next) {
-	GnomeVFSFileInfo * info = l->data;
-
-	if (strcmp (".", info->name) != 0 && strcmp ("..", info->name) != 0) {
-	  gchar * filename;
-
-	  filename = g_build_filename (datadir, info->name, NULL);
-	  gnome_wp_xml_load_xml (data, filename);
-	  g_free (filename);
-	}
-      }
-      gnome_vfs_file_info_list_free (list);
-
-      gnome_vfs_monitor_add (&handle, datadir, GNOME_VFS_MONITOR_DIRECTORY,
-			     (GnomeVFSMonitorCallback) gnome_wp_file_changed,
-			     data);
-    }
+    gnome_wp_xml_load_from_dir (datadir, data);
     g_free (datadir);
   }
   g_strfreev (xdgdirs);
 
-  if (g_file_test (WALLPAPER_DATADIR, G_FILE_TEST_IS_DIR)) {
-    gnome_vfs_directory_list_load (&list, WALLPAPER_DATADIR,
-				   GNOME_VFS_FILE_INFO_DEFAULT |
-				   GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-
-    for (l = list; l != NULL; l = l->next) {
-      GnomeVFSFileInfo * info = l->data;
-
-      if (strcmp (".", info->name) != 0 && strcmp ("..", info->name) != 0) {
-	gchar * filename;
-
-	filename = g_build_filename (WALLPAPER_DATADIR, info->name, NULL);
-	gnome_wp_xml_load_xml (data, filename);
-	g_free (filename);
-      }
-    }
-    gnome_vfs_file_info_list_free (list);
-
-    gnome_vfs_monitor_add (&handle, WALLPAPER_DATADIR, GNOME_VFS_MONITOR_DIRECTORY,
-			   (GnomeVFSMonitorCallback) gnome_wp_file_changed,
-			   data);
-  }
+  gnome_wp_xml_load_from_dir (WALLPAPER_DATADIR, data);
 
   gnome_wp_load_legacy (data);
 }
@@ -379,6 +413,7 @@ void gnome_wp_xml_save_list (AppearanceData *data) {
     GnomeWPItem * wpitem = list->data;
     const char * none = "(none)";
     gchar * filename;
+    const gchar * scale, * shade;
     gchar * pcolor, * scolor;
 
     if (!strcmp (wpitem->filename, none) ||
@@ -390,13 +425,15 @@ void gnome_wp_xml_save_list (AppearanceData *data) {
 
     pcolor = gdk_color_to_string (wpitem->pcolor);
     scolor = gdk_color_to_string (wpitem->scolor);
+    scale = wp_item_option_to_string (wpitem->options);
+    shade = wp_item_shading_to_string (wpitem->shade_type);
 
     wallpaper = xmlNewChild (root, NULL, (xmlChar *)"wallpaper", NULL);
     gnome_wp_xml_set_bool (wallpaper, (xmlChar *)"deleted", wpitem->deleted);
     item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"name", (xmlChar *)wpitem->name);
     item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"filename", (xmlChar *)filename);
-    item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"options", (xmlChar *)wpitem->options);
-    item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"shade_type", (xmlChar *)wpitem->shade_type);
+    item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"options", (xmlChar *)scale);
+    item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"shade_type", (xmlChar *)shade);
     item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"pcolor", (xmlChar *)pcolor);
     item = xmlNewTextChild (wallpaper, NULL, (xmlChar *)"scolor", (xmlChar *)scolor);
     g_free (pcolor);
